==> portmanager-0.3.4.tar.gz: portmanager-0.3.4/libMGPM/src/MGPMrMakeDescribe.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

#define PORTDIR_F0	0
#define PORTNAME_F1	1
#define MAKEFILESIZE_F2 2
#define MAKEFILETIME_F4 4

int	rGetFromMakeDescribe( structProperty* property, char* installedPortDir, char* available );

/*
	MGPMrMakeDescribe purpose:  
		given an installed port's directory returns its full name.
	
	Initially the portname is retrieved using:
	
		 "cd /usr/ports/--catagory--/--port directory--/; make -V PKGNAME"

	Because "make -V PKGNAME" is slow the retreived port's name and	date-time/size
	of it's Makefile are saved in /usr/local/share/portmanager/ports_cache.db.

	If MGPMrMakeDescribe is called a second time for the same port name and the port's 
	/usr/ports/--catagory--/--port directory--/Makefile has not been changed the portname
	is retrieved from ports_cache.db, wich is speedier than retrieving it by running
	"make -V PKGNAME".
*/
int	MGPMrMakeDescribe( structProperty* property, char* installedPortDir, char* available )
{
	char	id[]				= "MGPMrMakeDescribe";
	char	exact[]				= "exact";
	char*	strMakeFile			= NULL;
	char*	strMakeFileSize			= NULL;
	char*	strMakeFileSizeTemp		= NULL;
	char*	strMakeFileTime			= NULL;
	char*	strMakeFileTimeTemp		= NULL;
	int	cacheIdx			= 0;
	int	cacheQty			= 0;
	int	errorCode			= 0;

	strMakeFile	= (char*)calloc( 255, 1 );
	strMakeFileSize	= (char*)calloc( 255, 1 );
	strMakeFileTime	= (char*)calloc( 255, 1 );

	strcpy( strMakeFile, PORTSDIR );
	strcat( strMakeFile, installedPortDir );
	strcat( strMakeFile, "/Makefile" );

	property->cacheDb	= MGdbOpen( property->cacheDbFileName );

	cacheQty		= MGdbGetRecordQty( property->cacheDb );

	/*
	 * if Makefile is missing, deletre its cache record then return
	 */
	if( MGrIfFileExist( strMakeFile ) == 0 )
	{
		available[0]	= 0;
		/* 
		 * need to delete the cache record here
		 */
		MGdbGoTop( property->cacheDb );
		if( MGdbSeek( property->cacheDb, property->fieldCacheDbPortDir, installedPortDir, property->fieldCacheDbPortDir, exact ) ) 
		{
			cacheIdx		= MGdbGetRecno( property->cacheDb ) - 1;
			if( ( errorCode = MGdbDelete( property->cacheDb, cacheIdx ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGdbDelete", errorCode );
				return( 1 );
			}
			MGdbDestroy( property->cacheDb );
			free( strMakeFile );
			free( strMakeFileSize );
			free( strMakeFileTime );
			return( 0 );
		}
		MGdbDestroy( property->cacheDb );
		free( strMakeFile );
		free( strMakeFileSize );
		free( strMakeFileTime );
		return( 0 );
	}
	strMakeFileSizeTemp	= MGrIntToString( MGrFileSize( strMakeFile ) );
	strcpy( strMakeFileSize, strMakeFileSizeTemp );
	strMakeFileTimeTemp	= MGrIntToString( (int)MGrFileTime( strMakeFile ) );
	strcpy( strMakeFileTime, strMakeFileTimeTemp );

	cacheIdx	= 1;

	while( cacheIdx < cacheQty )
	{
		/* if installedPortDir (passed on calling MGPMrMakeDescribe) == a portDir in the cache db */
		if( strcmp( installedPortDir, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbPortDir ) ) == 0 )
		{
			/* and if the Makefile size for installedPortDir is the same size as in the cache db */
			if( strcmp( strMakeFileSize, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbMakeFileSize ) ) == 0 ) 
			{
				/* and if the Makefile time of creation is the same as in the cache db */
				if( strcmp( strMakeFileTime, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbMakeFileTime ) ) == 0 ) 
				{
					strcpy( available, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbAvailablePortName ) );
					MGdbDestroy( property->cacheDb );
					free( strMakeFileTime );
					free( strMakeFile );
					free( strMakeFileSize );
					return( 0 );
				}
				else
				{
					/* if here then Makefile time is wrong  so delete the cache db record */
					if( ( errorCode = MGdbDelete( property->cacheDb, cacheIdx ) ) )
					{
						fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGdbDelete", errorCode );
						return( 1 );
					}
					cacheIdx		= cacheQty;
					break;
				}
			}
			else
			{
				/* if here then Makefile size is wrong so delete the record */
				if( ( errorCode = MGdbDelete( property->cacheDb, cacheIdx ) ) )
				{
					fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGdbDelete", errorCode );
					return( 1 );
				}
				cacheIdx		= cacheQty;
				break;
			}
		}
		cacheIdx++;
	}
	rGetFromMakeDescribe( property, installedPortDir, available );
	MGdbAdd( property->cacheDb, installedPortDir, available, strMakeFileSize, strMakeFileTime, NULL );

	/*
	 * the cache record we just added will be the last one in the db
	 */
	available	= MGdbGet( property->cacheDb, MGdbGetRecordQty( property->cacheDb )-1, property->fieldCacheDbAvailablePortName );

	MGdbDestroy( property->cacheDb );
	free( strMakeFile );
	free( strMakeFileSize );
	free( strMakeFileTime );

	return( 0 );
}

/* get correct portname by running "make {options if any from pm-020.conf} describe" in portname's directory */
/* in version 0.3.0 switched from "make describe" to make -V PKGNAME for slight speed improvement */
int	rGetFromMakeDescribe( structProperty* property, char* installedPortDir, char* available )
{
	FILE*	tempStream		= NULL;
	char	id[]			= "rGetFromMakeDescribe";
	char*	currentPortMakefile	= NULL;
	int	idx			= 0;
	int	miscPtr 		= 0;
	int	skip			= 0;

	currentPortMakefile	= ( char* )calloc( 255, 1 );

	property->configDb		= MGdbOpen( property->configDbFileName );

	while( idx < MGdbGetRecordQty( property->configDb ) )
	{
		/* see if this port has anything special in configDb */
		if( strcmp( installedPortDir, MGdbGet( property->configDb, idx, property->fieldConfigDbKey ) ) == 0 )
		{
			strcpy( currentPortMakefile, "( cd " );
			strcat( currentPortMakefile, PORTSDIR );
			strcat( currentPortMakefile, installedPortDir );
			strcat( currentPortMakefile, "; make " );
			strcat( currentPortMakefile, MGdbGet( property->configDb, idx, property->fieldConfigDbValue ) );
			strcat( currentPortMakefile, " -V PKGNAME )" );
			skip	= 1;
			break;
		}
		idx++;
	}
	if( skip == 0 ) /* port had no entries in configure.db */
	{
		strcpy( currentPortMakefile, "( cd " );
		strcat( currentPortMakefile, PORTSDIR );
		strcat( currentPortMakefile, installedPortDir );
		strcat( currentPortMakefile, "; make " );
		strcat( currentPortMakefile, " -V PKGNAME )" );

	}
	tempStream	= popen( currentPortMakefile, "r" );
	if( ferror( tempStream ) )
	{
		fprintf( stderr, "%s error: reading make -V PKGNAME into available\n", id );
		perror( "system message" );
		assert(0);
	}
	fread( available, 254, 1, tempStream );
	pclose( tempStream );
	miscPtr = 0;
	while( miscPtr < (int)strlen( available ) )
	{
		if( available[miscPtr] == LINEFEED )
		{
			available[miscPtr] = 0;
			miscPtr = ( int )strlen( available );
		}
		miscPtr++;
	}
	MGdbDestroy( property->configDb );
	free( currentPortMakefile );
	return( 0 );
}

==> portmanager-0.4.1.tar.gz: portmanager-0.4.1/libMGPM/src/MGPMrMakeDescribe.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

#define PORTDIR_F0	0
#define PORTNAME_F1	1
#define MAKEFILESIZE_F2 2
#define MAKEFILETIME_F4 4

int	rGetFromMakeDescribe( structProperty* property, char* installedPortDir, char* available );

/*
	MGPMrMakeDescribe purpose:  
		given an installed port's directory returns its full name.
	
	Initially the portname is retrieved using:
	
		 "cd /usr/ports/--catagory--/--port directory--/; make -V PKGNAME"

	Because "make -V PKGNAME" is slow the retreived port's name and	date-time/size
	of it's Makefile are saved in /usr/local/share/portmanager/ports_cache.db.

	If MGPMrMakeDescribe is called a second time for the same port name and the port's 
	/usr/ports/--catagory--/--port directory--/Makefile has not been changed the portname
	is retrieved from ports_cache.db, wich is speedier than retrieving it by running
	"make -V PKGNAME".
*/
int	MGPMrMakeDescribe( structProperty* property, char* installedPortDir, char* available )
{
	char	id[]				= "MGPMrMakeDescribe";
	char	exact[]				= "exact";
	char*	strMakeFile			= NULL;
	char*	strMakeFileSize			= NULL;
	char*	strMakeFileSizeTemp		= NULL;
	char*	strMakeFileTime			= NULL;
	char*	strMakeFileTimeTemp		= NULL;
	int	cacheIdx			= 0;
	int	cacheQty			= 0;

	strMakeFile	= (char*)calloc( 255, 1 );
	strMakeFileSize	= (char*)calloc( 255, 1 );
	strMakeFileTime	= (char*)calloc( 255, 1 );

	MGmStrcpy( strMakeFile, PORTSDIR );
	MGmStrcat( strMakeFile, installedPortDir );
	MGmStrcat( strMakeFile, "/Makefile" );

	property->cacheDb	= MGdbOpen( property->cacheDbFileName );

	cacheQty		= MGdbGetRecordQty( property->cacheDb );

	/*
	 * if Makefile is missing, deletre its cache record then return
	 */
	if( MGrIfFileExist( strMakeFile ) == 0 )
	{
		available[0]	= 0;
		/* 
		 * need to delete the cache record here
		 */
		MGdbGoTop( property->cacheDb );
		if( MGdbSeek( property->cacheDb, property->fieldCacheDbPortDir, installedPortDir, property->fieldCacheDbPortDir, exact ) ) 
		{
			cacheIdx		= MGdbGetRecno( property->cacheDb ) - 1;
			property->cacheDb = MGdbDelete( property->cacheDb, cacheIdx );
			MGdbDestroy( property->cacheDb );
			free( strMakeFile );
			free( strMakeFileSize );
			free( strMakeFileTime );
			return( 0 );
		}
		MGdbDestroy( property->cacheDb );
		free( strMakeFile );
		free( strMakeFileSize );
		free( strMakeFileTime );
		return( 0 );
	}
	strMakeFileSizeTemp	= MGrIntToString( MGrFileSize( strMakeFile ) );
	MGmStrcpy( strMakeFileSize, strMakeFileSizeTemp );
	strMakeFileTimeTemp	= MGrIntToString( (int)MGrFileTime( strMakeFile ) );
	MGmStrcpy( strMakeFileTime, strMakeFileTimeTemp );

	cacheIdx	= 1;

	while( cacheIdx < cacheQty )
	{
		/* if installedPortDir (passed on calling MGPMrMakeDescribe) == a portDir in the cache db */
		if( strcmp( installedPortDir, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbPortDir ) ) == 0 )
		{
			/* and if the Makefile size for installedPortDir is the same size as in the cache db */
			if( strcmp( strMakeFileSize, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbMakeFileSize ) ) == 0 ) 
			{
				/* and if the Makefile time of creation is the same as in the cache db */
				if( strcmp( strMakeFileTime, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbMakeFileTime ) ) == 0 ) 
				{
					MGmStrcpy( available, MGdbGet( property->cacheDb, cacheIdx, property->fieldCacheDbAvailablePortName ) );
					MGdbDestroy( property->cacheDb );
					free( strMakeFileTime );
					free( strMakeFile );
					free( strMakeFileSize );
					return( 0 );
				}
				else
				{
					/* if here then Makefile time is wrong  so delete the cache db record */
					property->cacheDb = MGdbDelete( property->cacheDb, cacheIdx );
					cacheIdx		= cacheQty;
					break;
				}
			}
			else
			{
				/* if here then Makefile size is wrong so delete the record */
				property->cacheDb = MGdbDelete( property->cacheDb, cacheIdx );
				cacheIdx		= cacheQty;
				break;
			}
		}
		cacheIdx++;
	}
	rGetFromMakeDescribe( property, installedPortDir, available );
	MGdbAdd( property->cacheDb, installedPortDir, available, strMakeFileSize, strMakeFileTime, NULL );

	/*
	 * the cache record we just added will be the last one in the db
	 */
	available	= MGdbGet( property->cacheDb, MGdbGetRecordQty( property->cacheDb )-1, property->fieldCacheDbAvailablePortName );

	MGdbDestroy( property->cacheDb );
	free( strMakeFile );
	free( strMakeFileSize );
	free( strMakeFileTime );

	return( 0 );
}

/* get correct portname by running "make {options if any from pm-020.conf} describe" in portname's directory */
/* in version 0.3.0 switched from "make describe" to make -V PKGNAME for slight speed improvement */
int	rGetFromMakeDescribe( structProperty* property, char* installedPortDir, char* available )
{
	FILE*	tempStream		= NULL;
	char	id[]			= "rGetFromMakeDescribe";
	char*	currentPortMakefile	= NULL;
	int	idx			= 0;
	int	miscPtr 		= 0;
	int	skip			= 0;

	currentPortMakefile	= ( char* )calloc( 255, 1 );

	property->configDb		= MGdbOpen( property->configDbFileName );

	while( idx < MGdbGetRecordQty( property->configDb ) )
	{
		/* see if this port has anything special in configDb */
		if( strcmp( installedPortDir, MGdbGet( property->configDb, idx, property->fieldConfigDbKey ) ) == 0 )
		{
			MGmStrcpy( currentPortMakefile, "( cd " );
			MGmStrcat( currentPortMakefile, PORTSDIR );
			MGmStrcat( currentPortMakefile, installedPortDir );
			MGmStrcat( currentPortMakefile, "; make " );
			MGmStrcat( currentPortMakefile, MGdbGet( property->configDb, idx, property->fieldConfigDbValue ) );
			MGmStrcat( currentPortMakefile, " -V PKGNAME )" );
			skip	= 1;
			break;
		}
		idx++;
	}
	if( skip == 0 ) /* port had no entries in configure.db */
	{
		MGmStrcpy( currentPortMakefile, "( cd " );
		MGmStrcat( currentPortMakefile, PORTSDIR );
		MGmStrcat( currentPortMakefile, installedPortDir );
		MGmStrcat( currentPortMakefile, "; make " );
		MGmStrcat( currentPortMakefile, " -V PKGNAME )" );

	}
	tempStream	= popen( currentPortMakefile, "r" );
	if( ferror( tempStream ) )
	{
		fprintf( stderr, "%s error: reading make -V PKGNAME into available\n", id );
		perror( "system message" );
		assert(0);
	}
	fread( available, 254, 1, tempStream );
	pclose( tempStream );
	miscPtr = 0;
	while( miscPtr < (int)MGrStrlen( available ) )
	{
		if( available[miscPtr] == LINEFEED )
		{
			available[miscPtr] = 0;
			miscPtr = ( int )MGrStrlen( available );
		}
		miscPtr++;
	}
	MGdbDestroy( property->configDb );
	free( currentPortMakefile );
	return( 0 );
}
